#include <xc.h>
#include "encoder.h"
#include "sound.h"
#include "frq.h"

static T_UN_STATES encoderState;
unsigned char encoderSwitchCnt = 0;
unsigned char encoderSwitch = 0;
unsigned int encoderSwitchLongCnt = 0;
signed char cnt=0;
unsigned char cnt_timeout = 0 ;

// Encoder table
const T_ENC_STEP EN_StepTab[16] = {
    ENC_NO_STEP, // 00->00 do nothing
    ENC_RIGHT_STEP, // 00->01 right
    ENC_NO_STEP, // 00->10 left
    ENC_ERROR_STEP, // 00->11 invalid

    ENC_LEFT_STEP, // 01->00 left
    ENC_NO_STEP, // 01->01 do nothing
    ENC_ERROR_STEP, // 01->10 invalid
    ENC_NO_STEP, // 01->11 right

    ENC_NO_STEP, // 10->00 right
    ENC_ERROR_STEP, // 10->01 invalid
    ENC_NO_STEP, // 10->10 do nothing
    ENC_NO_STEP, // 10->11 left

    ENC_ERROR_STEP, // 11->00 invalid
    ENC_NO_STEP, // 11->01 left
    ENC_NO_STEP, // 11->10 right
    ENC_NO_STEP // 11->11 do nothing
};

void ENC_Init(void) {
    // Read the encoder
    encoderState.bits.new_A = PORTAbits.RA0;
    encoderState.bits.new_B = PORTAbits.RA1;
    encoderState.state.old = encoderState.state.new;
}

void ENC_Task(void) {

    // Read the encoder
    encoderState.bits.new_A = PORTAbits.RA0;
    encoderState.bits.new_B = PORTAbits.RA1;

    // debounce the switch;
    if (PORTAbits.RA3 == 0) {
        if (encoderSwitchCnt < 10)
            encoderSwitchCnt++;
        else {
            if (encoderSwitch != 1) {
                encoderSwitch = 1;
                FRQ_ToggleFine();
            }
        }
    } else {
        if (encoderSwitchCnt > 0)
            encoderSwitchCnt--;
        else {
            encoderSwitch = 0;
        }
    }

    if (encoderSwitch) {
        if (encoderSwitchLongCnt< 5000) {
            encoderSwitchLongCnt ++;
        }
        else {
            FRQ_Output();
            encoderSwitchLongCnt = 0;
        }
    }
    else
        encoderSwitchLongCnt = 0;


    // Decode rotary encoder
    switch (EN_StepTab[encoderState.index]) {
        case ENC_NO_STEP:
            break;
        case ENC_LEFT_STEP:
            cnt --;
            cnt_timeout = 0;
            break;
        case ENC_RIGHT_STEP:
            cnt ++;
            cnt_timeout = 0;
            break;
        case ENC_ERROR_STEP:
            break;
    }


    // If there were 2 steps, do not wait.
    // Count them immediatly
    if (cnt == 2) {
        cnt = 0;
        cnt_timeout = 0;
        FRQ_IncDec(INC);
        FRQ_IncDec(INC);
    }

    // If there were 2 steps, do not wait.
    // Count them immediatly
    if (cnt == -2) {
        cnt = 0;
        cnt_timeout = 0;
        FRQ_IncDec(DEC);
        FRQ_IncDec(DEC);
    }

    // Wait 10ms for "no change in signal A and B"
    if (cnt_timeout < 100) {
        cnt_timeout ++;
    }
    else {

        // 1 pulse and at least for x ms "no change"
        // now you can count them.
        if (cnt == 1 )
            FRQ_IncDec(INC);
        if (cnt == -1)
            FRQ_IncDec(DEC);

        cnt = 0;
   }

    // copy the new state into the old state for the next task
    encoderState.state.old = encoderState.state.new;

}
